# This file is overwritten during software install.
# Persistent customizations should go in a .local file.
include disable-passwdmgr.local

blacklist ${HOME}/.config/Bitwarden
blacklist ${HOME}/.config/KeePass
blacklist ${HOME}/.config/keepass
blacklist ${HOME}/.config/keepassx
blacklist ${HOME}/.config/keepassxc
blacklist ${HOME}/.config/Sinew Software Systems
blacklist ${HOME}/.fpm
blacklist ${HOME}/.keepass
blacklist ${HOME}/.keepassx
blacklist ${HOME}/.keepassxc
blacklist ${HOME}/.lastpass
blacklist ${HOME}/.local/share/KeePass
blacklist ${HOME}/.local/share/keepass
blacklist ${HOME}/.password-store
