# This file is overwritten during software install.
# Persistent customizations should go in a .local file.
include allow-php.local

noblacklist ${PATH}/php*
noblacklist /usr/lib/php*
noblacklist /usr/share/php*
