# This file is overwritten during software install.
# Persistent customizations should go in a .local file.
include allow-python3.local

noblacklist ${PATH}/python3*
noblacklist /usr/include/python3*
noblacklist /usr/lib/python3*
noblacklist /usr/local/lib/python3*
noblacklist /usr/share/python3*
